/*
  This program is free software published under the
  terms of the GNU Lesser General Public License.
  http://www.gnu.org/copyleft/lesser.html    
  This program is distributed in the hope that it will
  be useful - WITHOUT ANY WARRANTY; without even the
  implied warranty of MERCHANTABILITY or FITNESS FOR A
  PARTICULAR PURPOSE.
*/ 


/*  xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
     coded in ECMAScript 5 - JavaScript 5  (ES5, ECMAScript 2009)
    xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx */


//  xxxxxxxxxxxxxxx   wm11 variable declarations  xxxxxxxxxxxxxxxx

// These variables below can be directly modified by the user on the options form: 
                
var rows = 5;   // rox count in one page (screen) => row count

var cols= 5;   // shape count in one row  => column count
 
var timer = 60;     // Task time in seconds 

var duration = 5000;   // Target stimulus interval (ms)
 
var trefresh = 1000;    // All stimulus refresh interval (ms)

var probab = 0.75;   // probability of the target stimulus appearing on the screen

var pmin = 1;      // Minimum number of simultaneous occurrence of target stimuli

var pmax = 3;      // Maximum number of simultaneous occurrences of target stimuli

var parad = "2";   // Paradigm type: "0" => Horizontal-Vertical;  "1" => Clockwise-Counterclockwise; "2" =>Move-Static
                                                                                                               
var sync = false;   // synchronic move or not.  if true == The non-target and target stimulus will be moved at same time in same points (SYNCHRONIC movement); otherwise: no synchronization between stimulus 
                                                                       
var stype = "2";   // This is constant throughout the task: => [ "0" == Shape ; "1" == Color;  "2" == Nothing;] 

var emoji = false;   // stimulus generator. if (emoji) => draw E-moji, else draw geometric shapes

var tcolor= "1";   //  color type : "0" => Full random color;  "1" => Only basic colors;  "2" =>Only black color

var demo = true;       // if (demo == true) => show correct solution

var prg_type = "1";   //  type of information in Progress Bar => default: remaining Time

var store = false;      // false => this web application will not store results within client web browser.; true => This web application will store the results inside client web browser.
 

// Note: let and const are not used because of backward compatibility (coded in ECMAScript 5)
// Other internal global wm11 variables and constants:


var clock = 100;   // basic timer resolution in ms. Default = 100ms

var timer_id = null;   // timer_id is the identifier of the repeated action you want to cancel. This ID is returned from setInterval().

var counter = 0;      // global timer counter;  if (counter >= time) Timer will call theEnd()   

var r;      // shape radius (square base -width/height). shape width=shape height

var dx;    // offset in the x-axis direction (for drawing cells)

var dy;   // offset in the y-axis direction (for drawing cells)

var dirty = false;  // if  (dirty==true) => user changed any options in form 

var is_task = false;    // task in progress or not  (for timer)

var lastDraw;    // time of drawing the last stimulus for timer stimulus refresh

var cor = 0;   // total correct answers in task (sum)

var err = 0;  // total errors in task (sum)

var mis = 0;  // number of target stimuli not detected by the user = total missing values in task (sum) 

var max_score= 0;   // maximum possible score in task (maximum total correct answers in task)
      
/*  stim = jagged array= for each [shape] =>
[
  [0]=x (left); [1]=y (top); [2]= shape type;  [3] = color; [4] = target or not-target stimulus; [5] = selected or not selected; [6] = paradigm type; [7] = current position
]*/
var stim = [];  

var stim_b = [];         // canvas bounds => stim_b =>  [0] = left; [1] = top; [2] = right; [3]= bottom

var target_on = [];  //    The time when the target stimulus is turned on => [time] x [cell=shape]

var target_off = []; //     The time when the target stimulus is turned off  0> [time] x [cell=shape]

// array values constants for Emoji drawings:
// var arr = [8986, 8987, 9200, 9748, 9749, 9875, 9917, 9924, 9925, 9940, 9962, 9970, 9973, 9981, 9986, 9989, 9992, 9993, 9995, 10062, 11088, 127746, 127749, 127796, 127797, 127800, 127802, 127803, 127805, 127806, 127807, 127809, 127812, 127813, 127815, 127827, 127925, 127931, 127984, 128018, 128024, 128054, 128105, 128107, 128115, 128116, 128118, 128123, 128124, 128152 ];                 
var arr = [8986, 8987, 9200, 9748, 9749, 9875, 9917, 9924, 9925, 9940, 
           9962, 9970, 9973, 9981, 9986, 9989, 9992, 9993, 9995, 10062,
           11088, 127746, 127749, 127796, 127797, 127800, 127802, 127803, 127805, 127806,
           127807, 127809, 127812, 127813, 127815, 127827, 127925, 127931, 127984, 128018, 
           128024, 128054, 128105, 128107, 128115, 128116, 128118, 128123, 128124, 128152 ];                     

var s_max= 36;    // maximum shape type [36  or  arr.lenght-1]. Don't change this without changing dependent functions (see source code) 



//  xxxxxxxxxxxxxxx     wm11 functions  xxxxxxxxxxxxxxxx



// init html page [create/start Timer, check storage, show start panel..] 
function init() 
{       
  if (timer_id)  {   
    window.clearInterval(timer_id);  
  }  
  timer_id= setInterval(function () {wm11Timer()},clock);      
  if (isStorage() && storageAvailable('localStorage'))  {
    document.getElementById("store_group").disabled= false;
  }
  else  {
    document.getElementById("store_msg").textContent= "Your browser probably doesn't have support for the Web Storage Object or access to the Storage Object is disabled. Check your browser settings. In some browsers (IE11, Edge) Storage Object is not enabled when accessed in a local file (offline).";
    document.getElementById("store_group").disabled= true;  
    document.getElementById("clr_store").disabled= true;     // IE11 bug  => childrens are not auto disabled
    document.getElementById("store").disabled= true;         // IE11 bug => childrens are not auto disabled
  }              
  defPage();    
  showStart();
}


// before using web storage, check browser support for localStorage and sessionStorage
function isStorage()  
{
  if (typeof(Storage) !== "undefined") {
     return true;               //  localStorage/sessionStorage is supported     
  } 
  else {
    return false;           // no browser Web Storage support
  }  
}


// necessarily because of IE11 and Edge offline mode
function storageAvailable(type) 
{
  try  {
    return (window[type]) ? true: false;   
  }
  catch(e) {
    return false;
  }
}


// show start panel 
function showStart()  
{      
  document.getElementById("end").style.display= "none";
  document.getElementById("google_wrapper").style.display="none";
  document.getElementById("table_grp").style.display="none";  
  document.getElementById("start").style.display= "block";
}


// show forms with application options (settings)
// <!-- IMPORTANT: showOption() and hideOption() buttons must be outside form element -->
function showOption()   
{
  document.getElementById("hide_option").style.display ="block";
  document.getElementById("options").style.display= "block";
  document.getElementById("start").classList.remove("child");  // don't work in <= IE 9    
  document.getElementById("start_div").style.display="none";  // .disabled = true;   
  document.getElementById("show_option").style.display ="none";  
  document.getElementById("hide_option").style="text-align:center";  // bug? auto-remove style when element.style.display="none"
  document.getElementById("instructions").style.display ="none";     
  if (document.getElementById("store_group").disabled === false && localStorage.getItem("wm11-v1.0")) 
  {
    var cnt= JSON.parse(localStorage.getItem("wm11-v1.0")).length;                                                        
    document.getElementById("store_cnt").textContent=  "You have a total of " + cnt  + " wm11 results stored in your web browser";
  }
  dirty= false;
}


// <!-- IMPORTANT: showOption() and hideOption() buttons must be outside form element -->
// save to window global vars all user Options/Settings
function hideOption()  
{
  setVarPar();   
  
  document.getElementById("options").style.display="none"; 
  document.getElementById("start").classList.add("child");   
  document.getElementById("show_option").style.display ="block";   
  document.getElementById("start_div").style.display="block";        //.removeAttribute("disabled");  
  document.getElementById("hide_option").style.display ="none";   
  document.getElementById("start_div").style="text-align:center";  // bug?  (firefox auto remove class "center" when set style.display.none;)
  document.getElementById("show_option").style="text-align:center";   // bug?    
  // update instructions
  if (parad === "0")  {   //   Paradigm type: "0" => Horizontal-Vertical;    
    document.getElementById("ins_p0").style.display= "block"; 
    document.getElementById("ins_p1").style.display= "none"; 
    document.getElementById("ins_p2").style.display= "none";               
  }
  else if (parad === "1")  {  //   "1" => Clockwise-Counterclockwise;   
    document.getElementById("ins_p0").style.display= "none"; 
    document.getElementById("ins_p1").style.display= "block"; 
    document.getElementById("ins_p2").style.display= "none";               
  }
  else  {  //  "2" =>Move-Static   
    document.getElementById("ins_p0").style.display= "none"; 
    document.getElementById("ins_p1").style.display= "none"; 
    document.getElementById("ins_p2").style.display= "block";               
  }
  document.getElementById("ins_demo").style.display= demo ? "block": "none";    
  document.getElementById("ins_prg").style.display= (prg_type === "1") ? "inline": "none";  
  document.getElementById("instructions").style.display ="block";
       
  refMsg();  
}

       
// refresh variable research Options (update *_msg label) => give user short option HELP
function refOption(e)  
{   
  dirty=true;       
  var my_id = e.id, msg = document.getElementById(my_id + "_msg"), s = "Values within this range are allowed: ", s2= " with step= ";    
     
  switch (my_id)
  {
    
    case "rows":     
    case "cols":
      if (e.validity.valid)  {           
        msg.removeAttribute("class");       
        msg.textContent= "";       //msg.textContent=  e.value;
        cmpWidth();        
      }
      else  {
        msg.textContent= s + e.min + ".." + e.max + s2 + e.step;
        msg.setAttribute("class", "error");    // msg.className = 'error';
      }         
      break; 
                  
    case "timer":
      if (e.validity.valid)  {
        msg.textContent= stringTime(e.value); 
        msg.removeAttribute("class");          
      }
      else  {
        msg.textContent= s + e.min + ".." + e.max + s2 + e.step;
        msg.setAttribute("class", "error");
      } 
      break;                
    
    case "duration":
      if (e.validity.valid)  
      { // duration depends on trefresh. refresh time takes precedence over the duration of exposure to target stimuli                 
        var tmp= document.getElementById("trefresh").value;
        if (e.value < 3*tmp)  {
          e.value= 3*tmp;  // set maximum allowed value
          document.getElementById("duration_msg").textContent="Target stimulus interval must be at least 3 times greater than All stimulus refresh interval. Autocorrected";
        }
        else  {
          msg.textContent= "Target stimulus display time - also the maximum reaction time for the response"; 
          msg.removeAttribute("class");
        }
      }
      else  {
        msg.textContent= s + e.min + ".." + e.max + s2 + e.step;
        msg.setAttribute("class", "error");
      } 
      break;   
        
    case "trefresh":
      if (e.validity.valid)  
      {    //  refresh time takes precedence over the duration of exposure to target stimuli     
        msg.textContent= "This is refresh time for all target and non-target stimulus"; 
        msg.removeAttribute("class"); 
        var tmp= document.getElementById("duration").value;
        if (tmp < 3*e.value)  {    
          document.getElementById("duration").value=3*e.value;   // set maximum allowed value for duration 
          document.getElementById("duration_msg").textContent="Target stimulus interval must be at least 3 times greater than All stimulus refresh interval. Autocorrected";           
        } 
      }
      else  {
        msg.textContent= s + e.min + ".." + e.max + s2 + e.step;
        msg.setAttribute("class", "error");
      } 
      break;     
     
    case "probab":
      if (e.validity.valid)  {
        msg.textContent= "100=always a minimum of 1 target stimulus on the screen, 0=no target stimuli"; 
        msg.removeAttribute("class");          
      }
      else  {
        msg.textContent= s + e.min + ".." + e.max + s2 + e.step;
        msg.setAttribute("class", "error");
      } 
      break;                  
      
             
    case "pmin":
    case "pmax": 
      var min_msg=document.getElementById("pmin_msg");
      var max_msg=document.getElementById("pmax_msg");
      if (e.validity.valid)  {     
        if (parseInt(document.getElementById("pmin").value,10) > parseInt(document.getElementById("pmax").value, 10))  
        {   
          min_msg.textContent= "Minimum number of simultaneous occurrences of target stimuli must be less than or equal to Maximum number of simultaneous occurrence of target stimuli"; 
          min_msg.setAttribute("class", "error");  
          max_msg.textContent= "Maximum number of simultaneous occurrences of target stimuli must be greater or equal then Minimum number of simultaneous occurrence of target stimuli"; 
          max_msg.setAttribute("class", "error"); 
        }
        else  {
          min_msg.textContent= "";          
          min_msg.removeAttribute("class");  
          max_msg.textContent= "";          
          max_msg.removeAttribute("class");  
        } 
      }
      else  {
        msg.textContent= s + e.min + ".." + e.max + s2 + e.step; 
        msg.setAttribute("class", "error");
      }        
      break;      
          
    case "parad":            
      if (e.value === "0")  {    
        msg.textContent= "The target stimulus will move vertically (up-center-down). The non-target stimulus will move horizontally (left-center-right)";
        document.getElementById("sync").disabled= false;     
      }
      else if (e.value === "1")  {   
        msg.textContent=  "The target stimulus will move counterclockwise. The non-target stimulus will move clockwise"; 
        document.getElementById("sync").disabled= false;                  
      }
      else  {                                                                                                       
        msg.textContent= "The target stimulus will be static (no movement). The non-target stimulus will move randomly";
        document.getElementById("sync").value= "0";
        document.getElementById("sync").disabled= true;          
      }        
      break;
      
    case "sync":        
      msg.textContent=(e.value === "0")   ? "In most cases: the task is more difficult. There will be no time synchronization between stimuli" : "Generally speaking: the task is easier. There will be time synchronization between stimuli when possible"              
      break;      
         
    case "stype":       // This is constant throughout the task: => [ "0" == Shape ; "1" == Color;  "2" == Nothing;]          
      if (e.value === "0")  {    // shape is constant throughout the task
        msg.textContent= "Change only color in all stimulus"; 
        document.getElementById("emoji").value="0";   // shapes
        document.getElementById("emoji").disabled= false;         
      }
      else if (e.value === "1")  {     // color is constant throughout the task
        msg.textContent=  "Change only shape in all stimulus";
        document.getElementById("emoji").value="0";    // shapes
        document.getElementById("emoji").disabled= true;                  
      }
      else  {       // change shape and color => Nothing is constant throughout the task
        msg.textContent= "Change shape and color in all stimulus"; 
        document.getElementById("emoji").disabled= false;
        //document.getElementById("tcolor").value = "0";    // set auto default value  => Full random color      
      }        
      break;
         
    case "emoji":     
      if (e.value === "1")  {  
        msg.innerHTML= "Emoji will be displayed. You should see elephant here: " + "&#128024";            
      }             
      else  {            
        msg.textContent="Geometric shapes will be displayed.";       
      }
      break;
                     
    case "tcolor":                                                                                                                                            
      if (e.value === "0") {
        msg.textContent= "All stimuli will be displayed in all possible colors (when possible). Caution: If a color similar to white is generated, you will not be able to distinguish the drawing from the background color."
      }
      else if (e.value === "1") {
        msg.textContent= "All stimuli will be displayed only in basic html colors (when possible)";
      } 
      else  {
        msg.textContent= "All stimuli will be drawn in black only";
      }
      
      break;
              
    case "demo":    
      if (e.checked)  {
        msg.textContent= " (Yes - all target stimuli will have this background color)";
        msg.setAttribute("style", "background-color:#FFFFF0");   // background-color:ivory;
      }
      else  {
        msg.textContent=" (No)";
        msg.removeAttribute("style");
      }    
      break;
                             
    case "prg_type":                                                                                                            
      msg.textContent= (e.value === "0") ? "The progress bar will not be displayed during the task at the bottom of the screen" : "The remaining time until the end of the task will be displayed";
      break;   
      
    case "store":                                                                                                                                            
      msg.textContent= (e.value === "1") ? "This web application will store the results inside your web browser (no expiration date). Useful for group research" : "This web application will not store results within your web browser. Data is lost when the browser tab is closed";
      break;  
      
  }   // switch (my_id)
       
}


// set default research screen layout (rows and columns count)
function defPage()
{    
  var _rows= 3;  
  var _cols= 3;
  var k= Math.round(window.innerWidth/window.innerHeight);
  var res;   // res == default shape radius (shape width=shape height) 
  var row_max= document.getElementById("rows").max;
  var col_max= document.getElementById("cols").max; 
  do  
  {       
    res= getWidth(_rows,_cols);
    if (res <= 25 || _rows >= row_max || _cols >= col_max) break;
    _rows++;
    _cols+=k;
  }
  while (res > 25);   // set default shape radius around 25px                                                       
  window.rows=_rows;
  window.cols=_cols;
  document.getElementById("rows").value=window.rows;
  document.getElementById("cols").value=window.cols; 
  document.getElementById("shp_demo").style="line-height:" + res + "px";        // vertical-align text middle
  document.getElementById("shp_width").textContent= "Expected shape size (px):" + res;
  document.getElementById("shp_demo").style="display:inline-block;background-color:#ff0000;vertical-align:middle;width:" + res + "px;height:" + res + "px";    
}


// row = row count; col= column count
// return (compute) shape radius
function getWidth(r, c)  
{   
  var _dx = Math.floor(window.innerWidth/c);  
  var _dy = Math.floor((window.innerHeight - 25)/r); 
  return Math.floor(Math.min(_dx,_dy)/6);  // this is shape radius. shape == each stimulus    
}  


// compute (estimate) shape radius and update html/dom elements. shape = each stimulus in cell 
function cmpWidth()
{ 
  var r= parseInt(document.getElementById("rows").value,10);
  var c= parseInt(document.getElementById("cols").value, 10);     
  var res= getWidth(r, c);  // this is shape radius. shape == each stimulus     
  document.getElementById("shp_width").textContent= "Expected shape size (px):" + res;
  document.getElementById("shp_demo").style="line-height:" + res + "px";        // vertical-align text middle
  document.getElementById("shp_demo").style="display:inline-block;background-color:#ff0000;vertical-align:middle;width:" + res + "px;height:" + res + "px";  
}


// refresh main validator message for user defined OPTIONS (feedback to user)
function refMsg()   
{ 
  if ( optionsValid() )  {
    document.getElementById("is_valid").textContent=  (dirty===true) ? "All your settings have been successfully updated" : "";  
    document.getElementById("is_valid").style="text-align:center;color:#000000"; 
  }
  else  {
    document.getElementById("is_valid").textContent="Some of your options are incorrectly set. They were not updated. These settings are marked in red."; 
    document.getElementById("is_valid").style= "text-align:center;color:#ff0000;font-weight:bold"; // safari 5.1.7 bug? => this don't work
  } 
}


// check for validity.valid object in all input type="number" 
function optionsValid()  
{     
  var res= document.getElementById("rows").validity.valid && 
    document.getElementById("cols").validity.valid &&             
    document.getElementById("timer").validity.valid &&    
    document.getElementById("duration").validity.valid   &&  
    document.getElementById("trefresh").validity.valid &&   
    document.getElementById("probab").validity.valid &&                
    document.getElementById("pmin").validity.valid  &&
    document.getElementById("pmax").validity.valid &&
    (parseInt(document.getElementById("pmin").value,10) <= parseInt(document.getElementById("pmax").value, 10));                           
    return res;       // Firefox parser don't execute code ok without res var               
}


// start wm11 Task (start Research) 
function startTask(e)   
{       
  e.disabled= true;
  stimGenerator(); 
  e.disabled= false;   
  do  
    var ok= e.disabled;
  while (ok); // timer will wait for stimulus generator 
  document.getElementById("google_wrapper").style.display="none";
  document.getElementById("end").style.display= "none"; 
  document.getElementById("start").style.display= "none"; 
  document.getElementById("canvas").style.display="block";            
      
  document.getElementById("progress").style.display= "block";     
  document.getElementById("progress").setAttribute("style", "width:100%");   
  document.getElementById("time_left").textContent= "";
  if  (document.getElementById("prg_type").value === "1")  {
    document.getElementById("progress").className= "progress-bar progress-bar-danger progress-bar-striped";  // remaining time   
  }  
  document.getElementById("progress").setAttribute("style", "width:0");  // reset progress bar width
   
  drawStim();       // draw stimulus set   
 
  counter= 0;       // reset global timer counter 
  is_task= true;    // finally "activate" timer
}


// set application variable parameters in Options form
function setVarPar()  
{   
 
  if (document.getElementById("rows").validity.valid) rows= parseInt(document.getElementById("rows").value, 10);     // number of rows
    
  if (document.getElementById("cols").validity.valid) cols= parseInt(document.getElementById("cols").value, 10);       // number of columns
   
  if (document.getElementById("timer").validity.valid) timer= Math.round(parseInt(document.getElementById("timer").value, 10));   // Task time in seconds 
  
  if (document.getElementById("duration").validity.valid && document.getElementById("trefresh").validity.valid) 
  {     // duration must be >= 3 * refresh time  and < timer => this block is probably redundant
    var d= Math.round(parseInt(document.getElementById("duration").value, 10));  
    if (d >= 3 * Math.round(parseInt(document.getElementById("trefresh").value, 10)))  {
      duration= d;
      document.getElementById("duration_msg").textContent=  "Target stimulus display time - also the maximum reaction time for the response";
    }    
    else  {
      duration= 3 * Math.round(parseInt(document.getElementById("trefresh").value, 10));
      document.getElementById("duration_msg").textContent="Target stimulus interval must be at least 3 times greater than All stimulus refresh interval. Autocorrected";
    }
  }   
  
  if (document.getElementById("trefresh").validity.valid) trefresh= Math.round(parseInt(document.getElementById("trefresh").value, 10));  // All stimulus refresh interval (ms)
 
  if (document.getElementById("probab").validity.valid) probab= parseInt(document.getElementById("probab").value, 10)/100;  // Probability of the target stimulus appearing on the screen 
                                                                                                                                                                                                            
  if (document.getElementById("pmin").validity.valid && document.getElementById("pmax").validity.valid 
    && (parseInt(document.getElementById("pmin").value,10) <= parseInt(document.getElementById("pmax").value, 10) ) )   {  
      pmin= parseInt(document.getElementById("pmin").value, 10);        // Minimum number of simultaneous occurrence of target stimuli
      pmax= parseInt(document.getElementById("pmax").value, 10);      // Maximum number of simultaneous occurrences of target stimuli
  }
                                                                                                    
  parad = document.getElementById("parad").value;   // // Paradigm type: "0" => Horizontal-Vertical;  "1" => Clockwise-Counterclockwise; "2" =>Move-Static 
       
  sync= (document.getElementById("sync").value === "1") ? true : false;   // synchronic move or not. if true == The non-target and target stimulus will be moved at same time in same points (SYNCHRONIC movement); otherwise: no synchronization between stimulus
  
  stype = document.getElementById("stype").value;      // This is constant throughout the task: => [ "0" == Shape ; "1" == Color;  "2" == Nothing;] 
      
  emoji = (document.getElementById("emoji").value === "1") ? true : false;   // stimulus generator. if (emoji) => draw E-moji, else draw geometric shapes
                   
  tcolor= document.getElementById("tcolor").value;      //  color type : "0" => Full random color;  "1" => Only basic colors;  "2" =>Only black color
             
  demo= (document.getElementById("demo").checked) ? true: false;   // if (demo == true) => show correct solution  
  
  prg_type= document.getElementById("prg_type").value;   //  Progress Bar  (none or Time) . Type of information in Progress Bar 
  
  store= (document.getElementById("store_group").disabled === false && document.getElementById("store").value === "1") ? true : false;      // false => this web application will not store results within client web browser.; true => This web application will store the results inside client web browser.
  
  s_max= emoji ? (arr.length-1) : 36;
  
}


// set internal application research parameters
function setPar()  
{  
          
  stim_b= [];  // bounds for stimulus
  stim= [];   // there is only one "item" 
  target_on= [];
  target_off= [];
  cor= 0; 
  err= 0;
  mis= 0;
  max_score= 0; 
   
  // set trigger time intervals for target stimulus   
  var max= rows*cols-1;
  var end_time=1000*timer-duration;   // last stimulus display
  for (var time=0; time<=end_time; time+=duration)
  {                             
    if (Math.random() < probab) 
    {     // (probab == 0) => never execute this; (probab == 1) => always execute this block  
      target_on[time]=[];     // insert new empty jagged array slot      
      target_off[time+duration]=[];   // if (typeof (target_off[time+duration]) === "undefined") target_off[time+duration]=[];
      var cnt= getRndInt(pmin, pmax);  // number of simultaneous occurrence of target stimuli
      var sel= {}, cell;  // selected cells (rect, shapes)
      for (var i=1; i<=cnt; i++) 
      {
        if (cnt < max)  
        {     // minimum one cell in non-target stimulus
          do
            cell= getRndInt(0, max);
          while (typeof sel[cell] != "undefined");     // in this time point each cell can be selected only once
          sel[cell]=1;        // remember this choice (cell)        
        }
        else  {
          cell= i-1;    // This is special case: all cells are target. no need for random generator          
        }
        target_on[time][cell]=true; // set cell to start display target stimulus (turn on)        
        target_off[time+duration][cell]=true;  // set cell to end display target stimulus (turn off)        
        max_score++;       
      }
    }      
  } 
}


// main wm11 stimulus generator function
function stimGenerator()  
{ 
  setPar();  
      
  setPage();
         
  stim= setStim(stim_b[0], stim_b[1], stim_b[2], stim_b[3]);               
              
}


/* set canvas (viewport) choices (options) bounds        
   stim_b array =>  [0] = left; [1] = top; [2] = right; [3]= bottom  */                                                                                    
function setPage()  
{ 
  document.getElementById("footer").style.display="block";  
  document.getElementById("footer").height= "25";  
  var canvas= document.getElementById("canvas");     
  canvas.left= 0; 
  canvas.top= 0;
  canvas.width= window.innerWidth;       // stretch canvas to parent
  canvas.height= window.innerHeight - document.getElementById("footer").height;                                   
  // set window.r => shape radius  
  dx = Math.floor(canvas.width/cols);  
  dy = Math.floor(canvas.height/rows); 
  r= Math.floor(Math.min(dx,dy)/6);  // this is shape radius. shape == each stimulus  
  stim_b= [0, 0, canvas.width, canvas.height];  //  stim_b =>  [0] = left; [1] = top; [2] = right; [3]= bottom                                                                                                                     
  if (r < 5)  alert("Your screen resolution is probably not enough to distinguish the drawings that will be displayed.\nPossible solutions:\n  -reduce Row count to a smaller value Currently= " + rows + "\n  -reduce the Column count. Currently= " + cols);      
}

   
/* set stimulus and sample coordinates; generate stim-coord (x,y) and default properties (target/non-target, color, shapes type)
 divide screen section into raster equal segments (x-horizontal and y-vertical) 
 return array  stim coordinates =>  array index:  [0]=x (left); [1]=y (top); [2]= shape type;  [3] = color; [4] = target or not-target stimulus; [5] = selected or not selected; [6] = paradigm type; [7] = current position
 parameters:   stim_b array =>  [0] = _left; [1] = _top; [2] = _right; [3]= _bottom */     
function setStim(_left, _top, _right, _bottom) 
{   
  var x, y, mov=0, arr= [];    // arr = function result  
  var shape= getRndInt(0, s_max);  // get first one random shape type  
  var col= newColor();    // get first one random color based on window.tcolor  
  y= _top;
  for (var i=1; i<=rows; i++) 
  { // i == screen row == x     
    x=_left;
    for (var j=1; j<=cols; j++)  
    {  // j == screen column == y                                                   
      // stype = This is constant throughout the task: => [ "0" == Shape ; "1" == Color;  "2" == Nothing;]                                 
      if (stype === "2")  {  //  Nothing in const during task
        shape= getRndInt(0, s_max);    // get new random shape type for each cell (shape)        
        col= newColor();     // get new random color for each cell (shape)           
      }
      else if (stype === "1")  {  // color is const during task
        shape= getRndInt(0, s_max);    // get new random shape type for each cell (shape)    
      } 
      else   {   // shape is const during task  
        col= newColor();   // get new random color for each cell (shape)             
      }                        
      
      if (!sync || parad === "2") // else mov= 0;  
      {
        switch (parad)
        {   // Paradigm type: "0" => Horizontal-Vertical;  "1" => Clockwise-Counterclockwise; "2" =>Move-Static
          case "0":        
            mov= getRndInt(0,2);    // 3. positions
            break;
          case "1": 
            mov= getRndInt(0,3);     // 4. positions
            break;
          default:  
            mov= getRndInt(0,4);     // 5. positions
        }
      }
      arr.push([x, y, shape, col, false, false, parad, mov]);   // array index: [0]=x (left); [1]=y (top); [2]= shape type;  [3] = color; [4] = target or not-target stimulus; [5] = selected or not selected; [6] = paradigm type; [7] = current position      
      x+=dx; // new shape within row => add x
    } 
    y+=dy;   // new row (add y)
  } 
  return arr;  
}


// get new random color based on window.tcolor
// var tcolor color type : "0" => Full random color (default);  "1" => Only basic colors;  "2" =>Only black color
function newColor()
{
  if (tcolor === "0")  {
    return getRndColor();
  }
  else if (tcolor === "1")  {
    return getColor();   
  }
  else  {
    return "#000000";  
  }
}


// sets one of mostly basic html predefined colors
// this is mostly default set of 16 html Basic colors 
function getColor() 
{
  var clr = [    
    '#000000',     /* "black" */   
    '#FF0000',     /* "red"   */
    '#008000',     /* "green" */
    '#0000FF',     /* "blue" */   
    '#FFFF00',     /* "yellow" */     
    '#00FF00',     /* "lime" */     
    '#00FFFF',     /* "cyan" / "aqua" */     
    '#FF00FF',     /* magenta / fuchsia  */  
    '#C0C0C0',     /* "silver" */      
    '#800000',     /* maroon */     
    '#808000',     /* "olive" */  
    '#800080',     /* "purple" */    
    '#008080',     /* teal */      
    '#000080',     /* navy */    
    '#FFC0CB',     /* "pink" */           
    '#FFA500'      /* "orange" */ 
    ];      
  var rnd= getRndInt(0,15);  // array clr limits
  //var rnd=  Math.floor(Math.random()*16);   // 0..15
  return clr[rnd];
}


// get full random colors in hex format
function getRndColor() 
{
  var letters = '0123456789abcdef';
  var color = '#';
  for (var i=0; i<6; i++) {
    color += letters[Math.floor(Math.random() * 16)];     // returns a random integer from 0 to 15 
  }
  return color;
}  


// get random integer from min to max range
function getRndInt(min, max) {
  return Math.floor(Math.random() * (max - min + 1)) + min;
}


// application timer handler function
function wm11Timer()  
{      
  if (!is_task) return;  // nop, idle        
    
  // first check for OFF target stim => end of target stimulus  
  if (typeof target_off[counter] !== "undefined")
  {  
    for (var s in target_off[counter])  
    {  // s => shape => cell => rect
      if (target_off[counter][s] === true && stim[s][4] === true)  
      { // compute score on this shape        
        if (stim[s][5] === true)  {           
          cor++;       // target rect is selected (user select correct rect) 
        }
        /*else if (stim[s][5] === false)   {    // target rect is not selected    
          mis++;
        }*/
        stim[s][4]= false;        // this shape (rect) is no longer the target stimulus        
      }      
    }
  } 
  // then check for ON target stim => begin of target stimulus    
  if (typeof target_on[counter] !== "undefined")
  {  
    for (var s in target_on[counter])  
    {     // s => shape => cell => rect
      if (target_on[counter][s] === true)  {
        stim[s][4]= true;     // this shape (rect) becomes the target stimulus
      }     
    }
  } 
  
  if (typeof target_off[counter] !== "undefined" || typeof target_on[counter] !== "undefined")  {
    unselect();    
  } 
  
  var rest = 1000 * timer - counter;     // this is remaining time (ms) 
  if (rest <= 0)  {     // is end of task?        
    theEnd();
  }  

  counter+=clock;  
     
  if (Date.now() - lastDraw >= trefresh)  {
    drawStim();   // refresh all stimulus         
  }
  
  if (prg_type === "1")   {   // "1" => Time. display time on Progress Bar   
    rest= Math.round(rest/1000);  // restore seconds => this is remaining time (s) 
    var percent= Math.round(100*rest/timer);    
    document.getElementById("progress").setAttribute("style", "width:"+percent+"%");    // !important    
    document.getElementById("time_left").textContent= stringTime(rest);    
  } 
      
} 


// unselect all rect (shapes)
function unselect()
{
  for (var s in stim)  {
    stim[s][5]= false;
  }
}


// return integer into string hh:mm:ss time format
// input par: total == integer (time in seconds)
function stringTime(total) 
{
  var hh = Math.floor(total/(60*60));         
  var divisor_for_minutes = total % (60 * 60);
  var mm = Math.floor(divisor_for_minutes / 60);       
  var divisor_for_seconds = divisor_for_minutes % 60;
  var ss = Math.ceil(divisor_for_seconds);    
  // These lines ensure you always have two-digits. This formats your string to HH:MM:SS
  if (hh < 10)  {hh = "0"+hh;}     
  if (mm < 10) {mm = "0"+mm;}
  if (ss < 10) {ss = "0"+ss;} 
  return hh + ":" + mm + ":" + ss;
} 


// draw (repaint) stimulus    
function drawStim() 
{     
  var canvas= document.getElementById('canvas');
  var ctx = document.getElementById('canvas').getContext('2d'); 
  ctx.clearRect(0, 0, canvas.width, canvas.height);      // clear canvas             
      
  for (var s in stim)   
  {   // s= shapes number                                           
    ctx.beginPath();     // this section draw square border for each shape/emoji => rectangle 
    ctx.rect(stim[s][0], stim[s][1], dx, dy);   //  	context.rect(x,y,width,height);
    ctx.strokeStyle=  "#000000";     
    if (stim[s][5] || (demo && stim[s][4]) === true)    // [4]= target or not; [5] == selected or not 
    { // set background color for selected answer or set background color in demo mode for correct answer. you must call before drawShape() handler               
      // Gray90 == selected or Ivory == correct 
      ctx.fillStyle=  stim[s][5]  ?  "#E5E5E5" : "#FFFFF0";  // the selected answer takes precedence over the correct answer (background color)                        
      //ctx.strokeStyle= "#FF0000";
    }
    else  {
      ctx.fillStyle= "#FFFFFF";
      ctx.strokeStyle= "#000000";
    }  
    ctx.fill();             
    ctx.stroke();
             
    var x,y;  // current shape x,y center
    switch (stim[s][6])  // paradigm type
    {                                         
      case "0":
        // parad === "0" : [ target stimulus =>  moving up-center-down and non-target stimulus => moving left-center-right ]
        if (stim[s][4])  
        {
          // target stimulus => moving up-center-down  (3 positions)
          x= stim[s][0] + dx/2;
          if (stim[s][7] === 0)  {
            y= stim[s][1] + r;           
            stim[s][7]=1;    
          }
          else if (stim[s][7] === 1)  {
            y= stim[s][1] + dy/2;           
            stim[s][7]=2;    
          } 
          else  { //if (stim[s][7] === 2)    
            y= stim[s][1] + dy - r; 
            stim[s][7]=0;    
          }  
        }              
        else 
        {
          // non-target stimulus => moving left-center-right (3 positions)
          y= stim[s][1] + dy/2;
          if (stim[s][7] === 0)  {
            x= stim[s][0] + r;           
            stim[s][7]=1;    
          }
          else if (stim[s][7] === 1)  {
            x= stim[s][0] + dx/2;           
            stim[s][7]=2;    
          } 
          else if (stim[s][7] === 2)  {
            x= stim[s][0] + dx - r; 
            stim[s][7]=0;    
          }  
        }  
        break;
        
        case "1":  
          // parad === "1" => [target = dynamic counterclockwise movement and non-target = dynamic clockwise movement]. they start from same position (total: 4 positions)        
          if (stim[s][4])  
          { 
            //target stimulus = dynamic counterclockwise movement (total: 4 positions)
            if (stim[s][7] === 0)  {   // 00:00 clock
              x= stim[s][0] + dx/2; 
              y= stim[s][1] + r;           
              stim[s][7]=1;    
            } 
            else if (stim[s][7] === 1)  {   // 09:00 clock
              x= stim[s][0] + r; 
              y= stim[s][1] + dy/2;           
              stim[s][7]=2;    
            }
            else if (stim[s][7] === 2)  {   // 06:00 clock
              x= stim[s][0] + dx/2; 
              y= stim[s][1] + dy - r;           
              stim[s][7]=3;    
            }
            else  {  // 03:00 clock
              x= stim[s][0] + dx - r; 
              y= stim[s][1] + dy/2;           
              stim[s][7]=0;    
            }
          } 
          else
          { 
            // non-target stimulus = dynamic clockwise movement (total: 4 positions)
            if (stim[s][7] === 0)  {   // 00:00 clock
              x= stim[s][0] + dx/2; 
              y= stim[s][1] + r;           
              stim[s][7]=1;    
            } 
            else if (stim[s][7] === 1)  {   // 03:00 clock
              x= stim[s][0] + dx - r; 
              y= stim[s][1] + dy/2;           
              stim[s][7]=2;    
            }
            else if (stim[s][7] === 2)  {   // 06:00 clock
              x= stim[s][0] + dx/2; 
              y= stim[s][1] + dy - r;           
              stim[s][7]=3;    
            }
            else  {  // 09:00 clock
              x= stim[s][0] + r; 
              y= stim[s][1] + dy/2;           
              stim[s][7]=0;    
            }
          } 
          break;
          
          default:      // 5. positions
            if (stim[s][4])  
            {  // target stimulus = static (no move) =>  center within rect
              x= stim[s][0] + dx/2; 
              y= stim[s][1] + dy/2;   
            }
            else 
            {  // non-target stimulus => random moving/jump within rect
              var pos;    // new shape position
              do
                pos= Math.floor(Math.random()*5);   // range: 0..4  or use: cur= getRndInt(0,4);
              while (pos == stim[s][7]);   // must be different from previous position
              stim[s][7]= pos;   // update new shape position
              switch (pos) 
              {
                case 0:   // 00:00 clock                  
                  x= stim[s][0] + dx/2; 
                  y= stim[s][1] + r;              
                  break;
                case 1:  // 03:00 clock                  
                  x= stim[s][0] + dx - r; 
                  y= stim[s][1] + dy/2; 
                  break;
                case 2: // 06:00 clock
                  x= stim[s][0] + dx/2; 
                  y= stim[s][1] + dy - r;       
                  break;
                case 3: // 09:00 clock
                  x= stim[s][0] + r; 
                  y= stim[s][1] + dy/2; 
                  break; 
                case 4:  // center within rect
                  x= stim[s][0] + dx/2; 
                  y= stim[s][1] + dy/2;
                // break;           
              }  // inner switch(pos)
            }      
            // break;                               
    }   //  switch (stim[s][6])  // paradigm type
    
    if (emoji)  {
      drawEmoji(ctx, x, y, stim[s][2], stim[s][3]);   
    }
    else  {
      drawShape(ctx, x, y, stim[s][2], stim[s][3]);   
    }          
  }                                           
  lastDraw= Date.now();
    
}


/* draw all emoji
   ctx = canvas context
   x = x coord square center,
   y = y coord square center   
   shape = e-moji type  
   col = color (fillSyle and/or strokeStyle) */
function drawEmoji(ctx, x, y, shape, col)
{                  
  y+=r;                                                                     
  ctx.font = Math.round(1.5*r).toString() + "px monospace, Courier";   // https://developer.mozilla.org/en-US/docs/Web/CSS/font-family   https://www.w3schools.com/cssref/css_websafe_fonts.asp 
  ctx.textBaseline="bottom";       ctx.textAlign = "center";  
  // Firefox, Chrome, Edge, Opera: ignore ctx.fillStyle and  ctx.strokeStyle= col  
  ctx.fillStyle = col;   // fill color => this line must be (only necessary for: IE11, Safari) 
  ctx.strokeStyle= col;  // line color (outer border line color). (only necessary for: IE11, Safari)            
  // why using: "tmp" element? because  Backward compatibility.  why not:  String.fromCodePoint()  ?  see: https://caniuse.com/#search=fromCodePoint
  //ctx.fillText(String.fromCodePoint(arr[shape]), x, y);     // direct decimal to emoji. => don't work in IE11, Opera Mini,..
  document.getElementById("tmp").innerHTML="&#" + arr[shape]; 
  ctx.fillText(document.getElementById("tmp").innerHTML, x, y);     // end of draw point number within each Circle       
}


/* shape function dispatcher (handler)
   ctx = canvas context
   x = x coord rect center,
   y = y coord rect center,   
   shape = shape type => this argument is main for call different shapes.js functions  
   col = color (fillSyle and/or strokeStyle)
   shapes.js file must be included in DOM  (wm11.html)
*/ 
function drawShape(ctx, x, y, shape, col)
{
  ctx.fillStyle= col;    // fill color
  ctx.strokeStyle= col;  // line color   
  switch (shape)  
  {
    case 0:   // full circle
    case 1:   // 1/2 circle RIGHT oriented
    case 2:   // 1/2 circle DOWN oriented   
    case 3:   // 1/2 circle LEFT oriented
    case 4:   // 1/2 circle UP oriented 
    case 5:   // 3/4 circle with UP-RIGHT hole
    case 6:   // 3/4 circle with DOWN-RIGHT hole
    case 7:   // 3/4 circle with DOWN-LEFT hole 
    case 8:   // 3/4 circle with UP-LEFT hole                          
     if (shape === 0)  {                        
       circle(ctx, x, y, r, 0, 2*Math.PI, false, col);  // full circle 
     }        
     else if (shape === 1)  {
       circle(ctx, x, y, r, 1.5*Math.PI, 0.5*Math.PI, false, col); // 1/2 circle RIGHT oriented      
     }
     else if (shape === 2)  {
       circle(ctx, x, y, r, 0, Math.PI, false, col); // 1/2 circle DOWN oriented     
     } 
     else if (shape === 3)  {
       circle(ctx, x, y, r, 0.5*Math.PI, 1.5*Math.PI, false, col); // 1/2 circle LEFT oriented     
     } 
     else if (shape === 4)  {
       circle(ctx, x, y, r, Math.PI, 0, false, col);   // 1/2 circle UP oriented     
     }        
     else if (shape === 5)  {
       circle(ctx, x, y, r, 0, 1.5*Math.PI, false, col);   // 3/4 circle with UP-RIGHT hole     
     } 
     else if (shape === 6)  { 
       circle(ctx, x, y, r, 0, 0.5*Math.PI, true, col);   // 3/4 circle with DOWN-RIGHT hole     
     } 
     else if (shape === 7)  { 
       circle(ctx, x, y, r, 0.5*Math.PI, Math.PI, true, col);   // 3/4 circle with DOWN-LEFT hole     
     } 
     else if (shape === 8)  { 
       circle(ctx, x, y, r, 1.5*Math.PI, Math.PI, false, col);   // 3/4 circle with UP-LEFT hole     
     }        
     break;
                
    case 9:   // polygon 3 equal sides
    case 10:  // polygon 4 equal sides   
     (shape === 9) ? polygon(ctx, x, y, r, 3, col):  polygon(ctx, x, y, r, 4, col);       
     break;    
                  
    case 11:  //  xxxxxx
     square(ctx, x, y, r-2, col);  // only for square: window.r must be > 2 for shape selection and demo background fill style  
     break;  //   xxxxxx
             
    case 12:    // longer side (base) down
    case 13:    // longer side (base) up
     (shape === 12) ? trapezoid(ctx, x, y, r, col, true) : trapezoid(ctx, x, y, r, col, false) ;  // base down or up   
     break;
              
    case 14: // rectangle - horizontal line is longer then vertical
    case 15: // rectangle - vertical line is longer then horizontal 
     (shape === 14) ? rectangle(ctx, x, y, r, col, true): rectangle(ctx, x, y, r, col, false);              
     break;
                   
    case 16:
    case 17:
     if (shape === 16) {   // vertical ellipse
      var w= r, h= 2*r; // change this 2 param for PI/2 rotation
     }
     else {   // horizontal ellipse
      var w= 2*r, h= r; // change this 2 param for PI/2 rotation
     }
     ellipse(ctx, x - w / 2.0, y - h / 2.0, w, h, col);       
     break; 
                
    case 18: // cross => classic vertical cross
    case 19: // cross => diagonal cross (x) 
     (shape === 18) ? cross(ctx, x, y, r, col, true): cross(ctx, x, y, r, col, false);       
     break;   
             
    case 20:  //(base == true) => draw down-right rotated paralelogram; 
    case 21: // (base== false) => draw left-down rotated paralelogram
     (shape === 20) ? paralelogram(ctx, x, y, r, col, true): paralelogram(ctx, x, y, r, col, false);    
     break;
                               
    case 22:   // star 4-points
    case 23:   // star 5-points
    case 24:   // star 6-points     
     star(ctx, x, y, shape-18, r/2, r, col);       
     break;
                            
    case 25:  //  vertical UP arrow (0)
    case 26:  //  vertical DOWN arrow (1)
    case 27:  //  horizontal LEFT arrow (2)
    case 28:  //  horizontal RIGHT arrow (3)
     arrow(ctx, x, y, r, col, shape-25);       
     break;
            
    case 29:  
    case 30:
     var a= (shape === 29) ? [r, r, 0, 0] : [0, 0, r, r];    // up rect round radius (31) or down round rect radius (32)
     roundRect(ctx, x-r, y-r, 2*r, 2*r, a, col);    
    break;
       
    case 31:     //  mushroom UP oriented     
    case 32:     //  mushroom RIGHT oriented
    case 33:     //  mushroom DOWN oriented
    case 34:     //  mushroom LEFT orinted 
      mushroom(ctx, x, y, r, col, shape-31);
      break;
               
    case 35:  // polygon 5 equal sides
    case 36:  // polygon 6 equal sides      
      (shape === 35) ?  polygon(ctx, x, y, r, 5, col):  polygon(ctx, x, y, r, 6, col);       
      //break; 
                      
  }   // switch (shape) of
   
}  


// user selected rectangle (shape) on canvas 
// <canvas id="canvas" onclick="doAnswer(event)"></canvas>   
function doAnswer(e)  
{        
  var x= e.clientX, y= e.clientY; 
  // stim_b =>  [0] = left; [1] = top; [2] = right; [3]= bottom 
  if (x > stim_b[0] && x < stim_b[2] && y > stim_b[1] && y < stim_b[3])  
  {         
    var col = Math.floor((x - stim_b[0]) / dx);   
    var row = Math.floor((y - stim_b[1]) / dy);      
    if (row <= rows && col < cols)  
    {
      var p = row * cols + col;   // shape (rect) position within stim array          
      if (stim[p])
      {   
        stim[p][5]= (stim[p][5] === true) ? false : true;   // toggle (reset) answer if user choose already selected answer => [5] = selected or not selected        
        if (!stim[p][4])  {  // this is not target rect (choice)
          err= (stim[p][5]) ? err+1: err-1;
        }
        drawStim(); // repaint => update backgroundColor for (un)selected shapes
      }         
    }     
  }   
}


// end of task (research)
function theEnd()  
{      
  is_task= false;
  document.getElementById("canvas").style.display="none"; // this disable doAnswer() event
  document.getElementById("footer").style.display="none";
  document.getElementById("end").style.display="block";  
  mis= max_score - cor;                                                  
  document.getElementById("score").textContent= "Correct= " + cor + " Missing= " + mis + " Errors= " + err + " Maximum possible score=" + max_score;
  document.getElementById("no_google").textContent="";  
  document.getElementById("chart_title").style.display="none"; 

  try  {
    drawReport();  
  }
  catch (e)  {
    document.getElementById("no_google").textContent= "You must have an available internet connection to display graphs and tables. Make sure you are online."; 
  } 
  
  try 
  {
    if (store) {
      drawStorage();  
    }
  }        
  catch (e)  { 
    document.getElementById("table_grp").innerHTML= "Your browser probably doesn't have support for the Web Storage Object or access to the Storage Object is disabled. Check your browser settings. In some browsers (IE11, Edge) Storage Object is not enabled when accessed in a local file (offline)."; 
  }
           
}



/* async draw and display table and chart with user results (call google API)..
  (chart = PIE + table) => Displays the sum of: correct answers, errors, and missing values   */
function drawReport() 
{          
  document.getElementById("google_wrapper").style.display="block";  
  //    xxx  table = Displays the sum of: correct answers, errors, and missing values  xxx 
  var data = new google.visualization.DataTable();    // Create and populate the data table. 
  data.addColumn('string', 'Answer type');     // 1. col = ans array = must be string  
  data.addColumn('number', 'Sum');      // sum(correct answers, errors, missing values) = number = frequencies             
  data.addRow ( ["Correct", cor] );
  data.addRow ( ["Errors", err] ); 
  data.addRow ( ["Missing", mis] );                                      
  options= {      
    'width': '50%',    
    'allowHtml': true, 
    'sort': 'enable',  
    'alternatingRowStyle': true              
  };           
  var elem= document.getElementById("table");
  elem.style.display = "block"; 
  new google.visualization.Table(elem).draw(data, options); 
  //    xxxxxx      PIE chart  => data are same   => draw PIE chart and table for correct, errors, missing values   xxxxxx 
  options= {   
   is3D: true, 
   legend: 'none',
   pieSliceText: 'value',
   tooltip: {text:'value'},    
   width: '100%',
   chartArea: {width:'100%'}
  };          //title: 'Correct answers, errors and missing values',   
  elem = document.getElementById("chart");   
  elem.style.display = "block"; 
  new google.visualization.PieChart(elem).draw(data, options);     // Create and draw the visualization.     
  document.getElementById("chart_title").style.display="block";  
}


//  use window.localStorage in client browser
function drawStorage()   
{  
  document.getElementById("table_grp").style.display="block";
  var oldItems = JSON.parse(localStorage.getItem("wm11-v1.0")) || []; 
  var id= oldItems.length+1;   
  // set new "record" 
  var newItem = {
    'id': id,
    'date': new Date(),      
    'correct': cor, 
    'errors': err,       
    'missing': mis,   
    'max-score': max_score,
    'show-correct' : demo ? "Yes": "No",      
    'paradigm': parad,
    'task-time-s': timer,
    'target-time-ms':duration,
    'refresh-time-ms':trefresh,    
    'rows' : rows,
    'cols' : cols,
    'probability': probab.toFixed(3),    
    'min-paralel': pmin,
    'max-paralel': pmax,    
    'sync' : sync ? "Yes" : "No",
    'sytpe' : document.getElementById("stype").value, 
    'emoji': emoji ? "Yes" : "No", 
    'tcolor' : document.getElementById("tcolor").value,      
    'progress' : document.getElementById("prg_type").value             
  };

  oldItems.push(newItem);

  localStorage.setItem("wm11-v1.0", JSON.stringify(oldItems));

  createTableJSON(document.getElementById("table_grp"), oldItems); 
   
  document.getElementById("score").textContent+= ". (ID=" + id + ")";
 
}


// convert JSON data dynamically to HTML Table using JavaScript - Dynamically Create table with rows in JavaScript
// downloaded and adapted from source: https://www.encodedna.com/javascript/populate-json-data-to-html-table-using-javascript.htm
function createTableJSON(elem, my_json) 
{  
  // EXTRACT VALUE FOR HTML HEADER. 
  var col = [];
  for (var i = 0; i < my_json.length; i++) {
    for (var key in my_json[i]) {
        if (col.indexOf(key) === -1) {
            col.push(key);
        }
    }
  } 
  var table = document.createElement("table");
  table.setAttribute("class", "my_table");    // table.className = 'my_table';
  // CREATE HTML TABLE HEADER ROW USING THE EXTRACTED HEADERS ABOVE.
  var tr = table.insertRow(-1);                   // TABLE ROW.
  for (var i = 0; i < col.length; i++) {
      var th = document.createElement("th");      // TABLE HEADER.
      th.innerHTML = col[i];
      tr.appendChild(th);
  }
  // ADD JSON DATA TO THE TABLE AS ROWS.
  for (var i = 0; i < my_json.length; i++) 
  {
    tr = table.insertRow(-1);
    for (var j = 0; j < col.length; j++) {
      var tabCell = tr.insertCell(-1);
      tabCell.innerHTML = my_json[i][col[j]];
    }
  }
  // FINALLY ADD THE NEWLY CREATED TABLE WITH JSON DATA TO A CONTAINER.
  elem.innerHTML= "";
  elem.appendChild(table);
}


// clear wm11 web browser storage data
function clrStorage()  
{  
  var cnt= localStorage.getItem("wm11-v1.0")  ?  JSON.parse(localStorage.getItem("wm11-v1.0")).length : 0;                                                        
  if (cnt === 0)  {
    alert("You have no wm11 data to delete");
    return;
  }  
  if ( !window.confirm("You have a total of " + cnt  + " wm11 results stored in your web browser. Continue?") ) { 
    return;
  }                                             
  if (window.confirm("Delete total saved wm11 results:  "  + cnt + ". This action cannot be undone. Final confirmation?") ) 
  { 
   window.localStorage.removeItem("wm11-v1.0"); 
   document.getElementById("store_cnt").textContent= "All data has been deleted. No data";
  }  
}
